﻿/**
 *  一、流程须知：
 *      1、自定义按钮执行顺序，beforeSubmit → onClick
 *      2、自定义按钮不会做保存动作，如需保存，主动执行eform.formEngine.save(function (id, controls) {});  事件顺序 saveBefore → saveAfter
 *      3、beforeSubmit 回调参数action，就是流程按钮配置中的类型
 *      4、currentActivity流程节点信息，发起流程时，currentActivity是没有数据的，除了退回到发起人
 *      5、直属主管：父级职位下的所有人。部门负责人：所在部门主职位下的所有人。
 *  二、附件列表：
 *      1、查询条件：eform.recordId & 32位controlId
 */


/** 公共方法类 */
class ISO {
    /**
     *  流程状态 
     */
    FlowStatus = {
        Draft: 0,  // 草稿
        InApproval: 1,  // 审批中
        Completed: 2,  // 已完成
        Terminated: 3  // 已终止
    };

    /**
     *  文件状态
     */
    FileStatus = {
        ToBeEffective: 0,  // 待生效
        Effective: 1,  // 已生效
        Expired: 2,  // 已过期
        Obsoleted: 3,  // 已作废
        Historical: 4,  // 历史的
        PendingReview: 5  // 待复审
    };
    CNFileStatus = {
        0: '待生效',
        1: '已生效',
        2: '已过期',
        3: '已作废',
        4: '历史的',
        5: '待复审'
    };
    FileStatusColor = {
        0: '#409EFF',
        1: '#67C23A',
        2: '#999999',
        3: '#EE5E54',
        4: '#B65EC5',
        5: '#F6AC57'
    };

    /**
     *  文件子状态
     */
    FileSubStatus = {
        Normal: 0,  // 正常的（默认值）
        InRevision: 10,  // 修订中
        ReviewRevision: 11,  // 修订中（复审）
        Recheck: 20,  // 复审中
        Obsoleting: 30,  // 作废中
        RecheckObsoleting: 31,  // 作废中（复审）
        ReissueInProgress: 40  // 补发中
    };
    CNFileSubStatus = {
        0: '正常的',
        10: '修订中',
        11: '复审修订中',
        20: '复审中',
        30: '作废中',
        31: '复审作废中',
        40: '补发中'
    };

    /**
     *  记录文件状态
     */
    FileRecordStatus = {
        ToBeEffective: 0,  // 待生效
        Effective: 1,  // 已生效
        Historical: 2,  // 历史的
        UpVersion: 3,//已回滚
        Obsoleted: 4,  // 已作废
    }

    /**
     *  流程分类 
     */
    FlowCategory = {
        Add: 0,  // 新增
        Revision: 1,  // 修订
        Reissue: 2,  // 补发
        Obsolete: 3,  // 作废
        Recheck: 4,  // 复审
        PaperApplication: 5  // 纸本申请
    };

    /**
     *  流程类型 
     */
    ProcessType = {
        Add: 0,  // 新增
        Modify: 1,  // 修订
        Obsolete: 2,  // 作废
        ProcessReissue: 3,  // 流程补发
        NonProcessReissue: 4,  // 非流程补发
        PermApplication: 5,  // 权限申请
        PaperApplication: 6  // 纸本申请
    };

    /**
     *  作废类型 
     */
    ObsoleteType = {
        Obsolete: 0,  // 直接作废
        Recheck: 1  // 复审作废
    };

    /**
     *  打印状态 
     */
    CNPrintStatus = {
        0: '未打印',
        1: '打印中',
        2: '已完成'
    };

    /**
     *  打印状态 
     */
    PrintStatus = {
        NotPrinted: 0,
        Printing: 1,
        Completed: 2,
        Cancelled: 3,
        Error: 4
    };
    /**
     *  打印状态 
     */
    CNPrintStatus = {
        0: '未打印',
        1: '打印中',
        2: '已完成',
        3: '已取消',
        4: '打印失败'
    };

    /**
     *  回收状态 
     */
    RecyclingStatus = {
        ToBeRecycled: 0,
        Recycled: 1,
        Destroyed: 2
    };
    CNRecyclingStatus = {
        0: '待回收',
        1: '已回收',
        2: '已销毁'
    };

    /**
     *  文件流程管理类型 
     */
    ProcessManageType = {
        NoPreTrial: 0,  // 无预审
        PreTrialAddition: 1,  // 预审新增（一次预审）
        PreTrialAdditionAndAmendments: 2, // 预审新增&修订
        AdditionPreTrialandAmendments: 3   // 新增预审&修订
    };

    /**
     *  文件预审-流程状态 
    */
    FilePreFlowStatus = {
        Unused: 0,  // 未使用
        InApproval: 1,  // 审批中
        Completed: 2,  // 已完成
        Terminated: 3  // 已终止
    };

    /**
     *  文件预审-预审类型 
    */
    FilePreType = {
        FileAddition: 1,  // 文件新增
        DocumentRevision: 2,  // 文件修订
    };

    constructor(eform) {
        // ..todo
        this.eform = eform;

        let origin = location.origin;
        let index = origin.lastIndexOf(':');
        this.serviceHost = index > 4 ? `${origin.substr(0, index)}:8002` : `${window.origin}:8002`;

        this.extendReplace();
    };

    /**
     *  扩展replace方法 
     */
    extendReplace() {
        String.prototype.myReplace = function (f, e) {
            let reg = new RegExp(f, 'g');
            return this.replace(reg, e);
        };
    };

    /** 
     *  获取多个组件value，并返回object
     * @param controlIds 组件Ids
     * @returns object{}
     */
    getValues(controlIds) {
        var obj = {};
        $.each(controlIds, function (i, controlId) {
            obj[controlId] = iso.getValue(controlId);
        });

        return obj;
    };

    /** 
     *  获取多个组件text，并返回obj
     * @param controlIds 组件Ids
     * @returns object{}
     */
    getTextList(controlIds) {
        var obj = {};
        $.each(controlIds, function (i, controlId) {
            obj[controlId] = iso.getText(controlId);
        });

        return obj;
    };

    /** 
     *  获取组件value
     * @param controlId 组件Id
     */
    getValue(controlId) {
        return iso.eform(controlId).method('getValue');
    };

    /** 
     *  获取组件text
     * @param controlId 组件Id
     */
    getText(controlId) {
        return iso.eform(controlId).method('getText');
    };

    /** 
     *  组件赋值
     * @param controlId 组件Id
     * @param value 组件值
     */
    setValue(controlId, value) {
        iso.eform(controlId).method('setValue', value);
    };

    /**
     *  批量设置组件值 
     *  @param {Object} data
     */
    setValues(data) {
        for (var key in data) {
            iso.setValue(key, data[key]);
        }
    }

    /**
     * 重置组件值
     * @param controlIds 组件Ids
     */
    resetValues(controlIds) {
        $.each(controlIds, function (i, controlId) {
            iso.setValue(controlId, '');
        });
    }

    /** 
     *  隐藏组件
     *  @param controlIds 组件Id集合
     */
    bulkHide(controlIds) {
        $.each(controlIds, function (i, controlId) {
            iso.eform.hide(controlId);
        });
    };

    /**
     *  隐藏及重置组件数据 
     *  @param controlId 组件Id
     */
    hideAndReset(controlId) {
        iso.eform.hide(controlId);
        iso.eform(controlId).method('setValue', '');
    };

    /** 
     *  隐藏组件
     *  @param controlIds 组件Id集合
     */
    bulkHideAndReset(controlIds) {
        $.each(controlIds, function (i, controlId) {
            iso.eform.hide(controlId);
            iso.setValue(controlId, '');
        });
    };

    /**
     *  批量显示组件 
     *  @param controlIds 组件Id集合
     */
    bulkShow(controlIds) {
        $.each(controlIds, function (i, controlId) {
            iso.eform.show(controlId);
        });
    };

    /**
     *  批量设置只读
     *  @param controlIds 组件Id集合
     *  @description 如果区块中包含"成员选择"组件，此方法在解析前不适用，因为组件渲染复杂，设置只读会失败
     */
    bulkSetReadonly(controlIds, bool) {
        $.each(controlIds, function (i, controlId) {
            iso.eform.setReadonly(controlId, bool);
        });
    };

    /**
     *  表单设置只读 
     *  @param formWindow 表单所在的window
     */
    setFormReadonly(formWindow) {
        let blocks = edoc2Form.formParser.blocks._hash;
        for (var key in blocks) {
            iso.eform.setReadonly(key, true, iso.eform.objType.block);
        }
    }

    /**
     *  显示流程按钮（解析后执行）
     *  @param {object} options
     *  @param {string} key 流程按钮类型：initiate、approve、cancel
     *  @param {boolean} value 是否显示
     */
    showFlowButton(options) {
        for (var key in options) {
            if (options[key]) {
                $(`a[action="${key}"]`).show();
            } else {
                $(`a[action="${key}"]`).hide();
            }
        }
    };

    /**
     *  初始化流程按钮
     *  @description 表单解析前使用
     *  @param controlId 组件Id
     *  @param types 流程按钮操作类型
     */
    initFlowButton(controlId, types) {
        let actions = JSON.parse(iso.eform(controlId).method('getProperty', "action").value);
        $.each(types, function (i, type) {
            let index = actions.map(_ => _.type).indexOf(type);
            actions.splice(index, 1);
        });

        iso.eform(controlId).method('setProperty', 'action', JSON.stringify(actions));
    };

    /**
     *  删除操作列
     *  @param controlId 组件Id
     */
    removeOperateColumns(controlId) {
        iso.eform.setReadonly(controlId, true);
        let config = JSON.parse(iso.eform(controlId).method('getControl').getProperty('config').value);
        config.operationColumnWidth = '0';
        iso.eform(controlId).method('getControl').setProperty('config', JSON.stringify(config));
    };

    /**
     *  初始化操作列列宽
     *  @param controlId 组件Id
     *  @param width 操作列列宽
     */
    initOperateColumnWidth(controlId, width) {
        let config = JSON.parse(iso.eform(controlId).method('getControl').getProperty('config').value);
        config.operationColumnWidth = width;
        iso.eform(controlId).method('getControl').setProperty('config', JSON.stringify(config));
    };

    /**
     *  左补位
     *  @param value 需补位的数据
     *  @param totalWidth 补位后总长度
     *  @param paddingChar 补位符
     *  @example
     *  // return 006
     *  padLeft(6, 3, '0');
     */
    padLeft(value, totalWidth, paddingChar) {
        return new Array(totalWidth - `${value}`.length + 1).join(paddingChar) + value;
    };

    /**
     * 右补位
     *  @param value 需补位的数据
     *  @param totalWidth 补位后总长度
     *  @param paddingChar 补位符
     *  @example
     *  // return 600
     *  padLeft(6, 3, '0');
     */
    padRight(value, totalWidth, paddingChar) {
        return value + new Array(totalWidth - `${value}`.length + 1).join(paddingChar);
    };

    /** 加载静态列表
     * @param controlId 组件Id
     * @param list 数据
     */
    loadStaticList(controlId, list, valueKey, textKey) {
        if (list.length > 0) {
            iso.eform(controlId).method('loadData', list.map(function (item) {
                return {
                    value: item[valueKey],
                    text: item[textKey]
                }
            }));
        }
    };

    /**
     *  获取当前日期时间yyyy-MM-dd HH:mm:ss 
     */
    getCurrentDateTime() {
        return iso.eform.getSystemVariable('System_Var_CurrentDateTime');
    };

    /** 
     *  列表文件名超链接格式化
     * @param fileId 文件Id
     * @param fileName 文件名称
     * @param fileVerId 文件版本Id
     */
    linkFormat(fileId, fileName, fileVerId) {
        let href = `/preview.html?fileId=${fileId}`;
        if (fileVerId) {
            href += `&fileverid=${fileVerId}`;
        }

        return `<a class="fileName_link" href="${href}" target="_blank">${fileName}</a>`;
    };

    /**
     *  流程实例号超链接
     */
    workflowLinkFormat(incident, process) {
        return `<a href="javascript:;" class="fileName_link"
                    onclick="iso.showDialog({
                                width: '70%',
                                height: '70%',
                                title: '流程状态',
                                target: '/edoc2Flow-web/edoc-processInstance/getProcessDiagramTrace.do?processDefinitionId=${process}&processInstanceId=${incident}&lng=${iso.eform.lang}&r=${Math.round(new Date())}'
                            })">${incident}</a>`;
    };

    /**
     *  获取审批意见
     *  @param controlId 流程组件Id
     */
    getApprovalComments(controlId) {
        let tag = $(`li[controlid="${controlId}"] textarea`);
        if (tag.length === 0) {
            return '';
        }

        return tag.val().trim();
    };

    /** 
     *  流程表单详情按钮格式化
     * @param text 按钮名称
     * @param type 流程类型
     * @param processId 流程Id
    * @param taskId 任务Id
     * @param incidentId 实例号Id
     * @param businessKey 流程表单Id
     * @param state 状态
     */
    linkProcessFormat(text, type, processId, taskId, incidentId, businessKey, state) {
        return `<a class="fileName_linkover" onclick="isoumpProcessing('${type}','${processId}','${taskId}','${incidentId}','${businessKey}','${state}')">${text}</a>`;
    };

    /** 跳转流程表单详情
     * @param type 流程类型
     * @param processId 流程Id
     * @param taskId 任务Id
     * @param incidentId 实例号Id
     * @param businessKey 流程表单Id
     * @param state 状态
     */
    jumpProcessing(type, processId, taskId, incidentId, businessKey, state) {
        if (!taskId || taskId === 'undefined') {
            taskId = '';
        }
        if (!incidentId || incidentId === 'undefined') {
            incidentId = '';
        }

        let data = {
            taskType: type,
            processId: processId,
            taskId: taskId,
            incidentId: incidentId,
            businessKey: businessKey,
            state: state
        };
        let token = $.cookie('token');

        $.ajax({
            type: 'get',
            url: `/edoc2Flow-web/edoc2-form/jumpIndex?orgToken=${token}&t=${new Date().getTime()}`,
            async: false,
            data: data,
            success: function (res) {
                eval(res);
            },
            error: function () {
                iso.layerAlert('error', '获取流程信息失败');
            }
        });
    };

    /**
     *  根据taskId查询当前节点任务信息
     *  @param taskId 任务Id
     */
    getCurrentActivity(taskId) {
        let result;

        if (!taskId) {
            taskId = $.getQueryString('taskId');
        }

        if (taskId) {
            $.ajax({
                url: `/edoc2Flow-web/edoc-task/${taskId}/getActivityInfo?taskId=${taskId}&orgToken=${$.getToken()}`,
                dataType: 'json',
                async: false,
                success: function (res) {
                    result = res;
                }
            });
        }

        return result;
    };

    /** 
     *  获取流程任务状态图
     * @param incidentId 实例号Id
     * @param taskState 任务状态
     */
    getTaskImage(incidentId, taskState) {
        if (!incidentId) {
            return '';
        }

        var stateImg = '';
        var title = '';

        switch (taskState) {
            case 0:
                stateImg = "normal.png";
                title = Edoc2FormSR.Flow_taskState_normal;

                break;
            case 5:
                stateImg = "backgo.png";
                title = Edoc2FormSR.Flow_taskState_backGo;

                break;
            case 10:
                stateImg = "back.png";
                title = Edoc2FormSR.Flow_taskState_back;

                break;
            case 20:
                stateImg = "complete.png";
                title = Edoc2FormSR.Flow_taskState_normal;

                break;
            case 30:
                stateImg = "timeout.png";
                title = Edoc2FormSR.Flow_taskState_timeout;

                break;
            case 50:
                stateImg = "assign.png";
                title = Edoc2FormSR.Flow_taskState_assign;

                break;
            case 100:
                stateImg = "normal.png";
                title = Edoc2FormSR.Flow_taskState_autoComlepte;

                break;
            case 110:
                stateImg = "normal.png";
                title = Edoc2FormSR.Flow_taskState_autoJump;

                break;
            case 120:
                stateImg = "normal.png";
                title = Edoc2FormSR.Flow_taskState_revoke;

                break;
            case 130:
                stateImg = "plusSign.png";
                title = Edoc2FormSR.Flow_taskState_plusSign;

                break;
            default:
                stateImg = "complete.png";

                break;
        }

        return `<img class="state_img" title="${title}" src="${edoc2Form.edoc2EformPath}/content/workflow/images/${stateImg}" />`;
    };

    /** 
     * 获取流程日志图
     * @param incidentId 实例号Id
     * @param processId 流程Id
     */
    getWorkflowLogImage(incidentId, processId) {
        let img = `<img class="workflowLog_img" title="${Edoc2FormSR.Control_WFLog}" src="${edoc2Form.edoc2EformPath}/content/workflow/images/workflow.png"`;

        return `<a onclick="iso.showWorkflow('${incidentId}', '${processId}')">${img}</a>`;
    };

    /**
     *  获取分配权限对象
     *  @description 根据分发单位
     *  @param {Object[]} distributions 分发单位
     *  @param {string} distributions[].MemberId 分发对象Id
     *  @param {string} distributions[].MemberType 分发对象类型
     *  @returns {Object[]} permissions
     *  @returns {string} permissions[].MemberId 分发对象Id
     *  @returns {string} permissions[].MemberType 分发对象类型
     *  @returns {string} permissions[].PermCateId 权限分类
     */
    getPermissionObject(distributions) {
        let permissions = [];

        $.each(distributions, function (i, d) {
            let datasetName = '';

            switch (parseInt(d.MemberType)) {
                case 0:  // 用户
                    datasetName = 'DB_GetUserById';

                    break;
                case 3:  // 组织
                    datasetName = 'DB_GetGroupById';

                    break;
                case 4:  // 职位
                    datasetName = 'DB_GetPositionById';

                    break;
                case 5:  // 部门
                    datasetName = 'DB_GetDepartmentById';

                    break;
            }

            iso.eform.dataset(datasetName, { id: d.MemberId }, function (res) {
                if (res.ResultCode == 0 && res.Data[0].length > 0) {
                    if (permissions.filter(_ => _.MemberId == res.Data[0][0].identityId).length === 0) {
                        permissions.push({
                            MemberId: res.Data[0][0].identityId,
                            MemberType: isoIO.MemberTypeObj[d.MemberType],
                            PermCateId: parseInt(d.Permission)
                        });
                    }
                }
            }, false);
        });

        return permissions;
    };

    /**
     *  分配对象
     *  @param recordId 表单Id
     *  @param processType 流程类型
     *  @param isPermApplication 是否权限申请
     */
    distributeUnit(recordId, processType, isPermApplication) {
        let result;
        iso.ajax({
            url: 'Distribution/UpdateDistributionUnit',
            data: JSON.stringify({
                RecordId: recordId,
                ProcessType: processType,
                IsPermApplication: !isPermApplication ? '0' : isPermApplication
            }),
            async: false,
            success: function (res) {
                if (res.ErrCode != 200) {
                    iso.layerAlert('error', '分发单位失败');
                }

                result = res;
            }
        });

        return result;
    }

    /** 
     *  查看流程日志
     * @param incidentId 实例号Id
     * @param processId 流程Id
     */
    showWorkflow(incidentId, processId) {
        let options = {
            width: '70%',
            height: '80%',
            title: '流程日志',  // 流程日志
            target: `/edoc2Flow-web/edoc-processInstance/getNewProcessDiagramTrace.do?processDefinitionId=${processId}&processInstanceId=${incidentId}&r=${new Date().getTime()}&lng=${iso.eform.lang}`
        };

        iso.showDialog(options);
    };

    /**
     *  获取流程token 
     */
    getWorkflowToken() {
        let token;

        $.ajax({
            url: `${window.origin}/edoc2Flow-web/login/index?appId=edoc2&password=edoc2`,
            dataType: 'json',
            async: false,
            success: function (res) {
                if (res.isSuccess) {
                    token = res.token;
                }
            },
            error: function () { }
        });

        return token;
    };

    /**
     *  发起复审流程
     */
    reviewStartProcess(documentFlowId) {
        let result = true;

        // 复审类型：0延用、1修订、2作废
        let reviewType = iso.getValue('ReviewType');
        let fileInfo = eval(iso.getValue('FileInfo'))[0];

        let flowToken = iso.getWorkflowToken();
        if (!flowToken) {
            iso.layAlert('error', `发起${reviewType === '1' ? '修订' : '作废'}复审流程失败`);
            return false;
        }

        let dsName = reviewType === '1' ? 'ISO文件修订流程' : '文件作废流程';
        let workflow = isoDataSet.getNewestWorkflow(dsName)[0];
        if (!workflow) {
            console.error('获取修订流程信息失败');
            iso.layAlert('error', `发起${reviewType === '1' ? '修订' : '作废'}复审流程失败`);
            return false;
        }

        let documentFlowInfo;
        iso.ajax({
            url: `DocumentFlow/CopyDocumentFlow?documentFlowId=${documentFlowId}&category=${reviewType === '1' ? iso.FlowCategory.Revision : iso.FlowCategory.Obsolete}&autoStart=true`,
            async: false,
            success: function (res) {
                if (res.ErrCode != 200) {
                    result = false;
                    iso.layerAlert('error', `发起${reviewType === '1' ? '修订' : '作废'}复审流程失败`);
                    return;
                }

                documentFlowInfo = res.Data;
            }
        });

        let context = '';
        if (documentFlowInfo) {
            let starter = reviewType == '1' ? eval(iso.getValue('Reviser'))[0].id : iso.getValue('ApplicantId');  // 修订人

            let remark = `${fileInfo.fileName}(复审)`;
            let requestParams = {
                processDefinitionId: workflow.processId,
                starter: starter,
                summary: remark,
                businessKey: documentFlowInfo.Id
            };

            let dccManagers = isoDataSet.getDCCManagers();
            if (dccManagers.length === 0) {
                iso.layerAlert('error', '暂无DCC文控管理审批人失败');
                return false;
            }

            //var userIds = dccManagers.map(_ => _.userId).join();
            let variablesParams = [{
                name: 'AutoStart',
                value: 1
            }];

            context = iso.startProcessInstance(requestParams, variablesParams, flowToken);
            if (!context) {
                iso.layerAlert('error', '流程发起失败');
                result = false;
            }
        }

        return result;
    }

    /**
     *  创建流程示例
     *  @param {object} requestParams 请求参数
     *  @param {string} requestParams.processDefinitionId 流程号
     *  @param {string} requestParams.starter 发起人Id
     *  @param {string} requestParams.summary 流程名称
     *  @param {string} requestParams.businessKey 业务id
     *  @param {Object[]} variablesParams 流程变量
     *  @param {string} variablesParams[].name 变量名称
     *  @param {string} variablesParams[].value 变量值
     */
    startProcessInstance(requestParams, variablesParams, token) {
        let context;

        $.ajax({
            url: `${window.origin}/edoc2Flow-web/edoc-processInstance/startProcessInstance`,
            type: 'post',
            data: {
                token: token,
                requsetParams: JSON.stringify(requestParams),
                variablesParams: JSON.stringify(variablesParams)
            },
            async: false,
            contentType: 'application/x-www-form-urlencoded',
            success: function (res) {
                if (res.isSuccess) {
                    context = res.context;
                }
            }
        });

        return context;
    };

    /**
     *  创建表格请求
     *  @param controlId 组件Id
     *  @param requestUrl 数据请求接口地址
     *  @param errMsg 接口内部错误消息
     *  @param keys 请求参数key
     *  @param params 请求参数
     *  @param callback 回调函数
     */
    createGridRequest(controlId, requestUrl, errMsg, keys, params, callback) {
        let grid = iso.eform(controlId);
        let requestParams = params ?? {};

        grid.method('onBeforeRequest', function (postData) {
            $.each(keys, function (i, key) {
                requestParams[key] = postData[key];
            });

            requestParams['Size'] = postData.size;
            requestParams['PageIndex'] = postData.index;
            requestParams['OrderType'] = postData.order_Type;
            requestParams['OrderField'] = postData.order_Field;

            return postData;
        });
        grid.method('onDataReceived', function (data) {
            iso.ajax({
                url: requestUrl,
                data: JSON.stringify(requestParams),
                async: false,
                success: function (res) {
                    if (res.ErrCode != 200) {
                        return iso.layerAlert('error', (errMsg ?? res.ErrMsg));
                    }

                    if ($.isFunction(callback)) {
                        callback(res);
                    }

                    data.rows = res.Data;
                    data.total = res.TotalCount;
                }
            });

            return data;
        });
    };

    /**
     *  获取审批领导，包含直属领导、部门领导
     *  @param userInfo 用户信息，如果不存在，则取当前用户信息
     */
    getLeaders(userInfo) {
        if (!userInfo) {
            userInfo = iso.eform.userInfo;
        }
        if (userInfo.MainPositionId === isoDataSet.getTopPositionId() || userInfo.MainDepartmentId === isoDataSet.getTopDeptId()) {
            return {
                leaders: [{
                    identityId: userInfo.IdentityId || userInfo.identityId
                }],
                deptLeaders: [{
                    userId: userInfo.ID
                }],
                positionLeaders: [{
                    userId: userInfo.ID
                }]
            };
        }

        let positionLeaders = [],
            deptLeaders = [],
            leaders = [];
        let dccManagers = isoDataSet.getDCCManagers();

        // 如果没有直属领导，则DCC文控管理审批
        deptLeaders = isoDataSet.getLeaders('Department', userInfo.MainDepartmentId);
        if (deptLeaders.length > 0) {
            leaders = deptLeaders;
        } else {
            deptLeaders = dccManagers;
            leaders = dccManagers;
        }

        positionLeaders = isoDataSet.getLeaders('Position', userInfo.MainPositionId);
        if (positionLeaders.length === 0) {
            positionLeaders = dccManagers.length === 0 ? isoDataSet.getDCCManagers() : dccManagers;
        }
        leaders = leaders.concat(positionLeaders);
        leaders = leaders.concat(dccManagers);

        if (leaders.length > 0) {
            let obj = {};
            leaders = leaders.reduce(function (item, next) {
                obj[next.identityId] ? '' : obj[next.identityId] = true && item.push(next);
                return item;
            }, []);
        }

        return {
            leaders: leaders,
            deptLeaders: deptLeaders,
            positionLeaders: positionLeaders
        };
    };

    /**
     *  初始化策略文件夹树 
     *  @param controlId 组件Id
     
    initStrategyTree(controlId) {
        iso.eform.dataset('DB_GetFolderByName', {
            folderName: 'ISO目录'
        }, function (res) {
            if (res.ResultCode != 0 || res.Data[0].length === 0) {
                return;
            }

            let treeData = [{
                id: res.Data[0][0].FolderId,
                folderId: res.Data[0][0].FolderId,
                text: res.Data[0][0].FolderName,
                children: []
            }];

            iso.eform.dataset('DB_GetStrategyFolders', {}, function (res2) {
                if (res2.ResultCode != 0 || res2.Data[0].length === 0) {
                    return;
                }

                let data = res2.Data[0];
                if (data.length > 0) {
                    treeData[0] = getChildren(treeData[0], data);
                }
            }, false);

            // 延迟加载，兼容火狐浏览器加载数据
            setTimeout(function () {
                iso.eform(controlId).method('getControl').easyTree.tree('loadData', treeData);
            }, 1);
        }, true);

        function getChildren(parent, data) {
            let children = data.filter(_ => _.Type === '0' && _.ParentFolderId == parent.folderId).map(function (d) {
                return {
                    id: d.Id,
                    folderId: d.FolderId,
                    text: d.Name,
                    state: 'closed',
                    children: []
                };
            });

            $.each(children, function (i, child) {
                let types = data.filter(_ => _.Type === '0' && _.ParentFolderId == child.folderId);  // 类型标识
                if (types.length > 0) {
                    getChildren(child, data);
                    parent.children.push(child);
                } else {
                    let strategies = data.filter(_ => _.Type === '1' && _.ParentFolderId == child.folderId).map(function (d) {  // 策略
                        return {
                            id: d.Id,
                            folderId: d.FolderId,
                            text: d.Name,
                            state: 'closed',
                            children: []
                        };
                    });

                    if (strategies.length > 0) {
                        child.children = strategies;
                        parent.children.push(child);
                    } else {
                        parent.children.push(data.filter(_ => _.Type === '0' && _.FolderId == child.folderId).map(function (d) {  // 类型标识
                            return {
                                id: d.Id,
                                folderId: d.FolderId,
                                text: d.Name,
                                state: 'closed',
                                children: []
                            };
                        })[0]);
                    }
                }
            });

            return parent;
        }
    };
*/
    /**
     *  接收打印客户端消息 
     *  @param json 客户端返回的消息
     */
    printSocketOnMessage(json) {
        let message = JSON.parse(json);

        isoDataSet.insertPrintRecord(message.Data);  // 新增打印记录
    };

    /**
     *  toast弹窗提示
     *  @param type 提示类型
     *  @param content 提示内容
     */
    layerAlert(type, content) {
        $.messager.layer({
            type: type,
            content: content
        });
    };

    /**
     *  根据分配对象查询所有人员信息 
     *  @param referenceId eform_ison_distribution.ReferenceId
     */
    queryMembersByDistribution(referenceId) {
        let distributionUnitList;
        iso.eform.dataset('DB_GetDistributionUnitByReferenceId', {
            referenceId: referenceId
        }, function (res) {
            distributionUnitList = res.Data[0];

            if (res.ResultCode != 0 || distributionUnitList.length === 0) {
                return iso.layerAlert('error', '当前打印任务未分配下发对象');
            }
        }, false);

        if (distributionUnitList.length === 0) {
            return [];
        }

        let members = [];
        var userIds = distributionUnitList.filter(_ => _.MemberType == 0).map(_ => _.MemberId);
        if (userIds.length > 0) {
            iso.eform.dataset('DB_QueryUsersInUserIds', {
                edoc2inarea_userIds: userIds.join()
            }, function (res) {
                if (res.ResultCode == 0 && res.Data[0].length > 0) {
                    members = res.Data[0].map(function (d) {
                        return {
                            Account: d.Account,
                            Name: d.Name,
                            Email: d.Email
                        };
                    });
                }
            }, false);
        }

        var groupIds = distributionUnitList.filter(_ => _.MemberType == 3);  // 用户组
        if (groupIds.length > 0) {
            iso.eform.dataset('DB_GetMembersByGroupIds', {
                edoc2inarea_groupIds: groupIds.join()
            }, function (res) {
                if (res.ResultCode == 0 && res.Data[0].length > 0) {
                    members.concat(res.Data[0].map(function (d) {
                        return {
                            Account: d.Account,
                            Name: d.Name,
                            Email: d.Email
                        };
                    }));
                }
            }, false);
        }

        $.each(distributionUnitList.filter(_ => _.MemberType == 4), function (i, d) {  // 职位
            iso.dmsAjax({
                url: `OrgPosition/GetUserListByPositionId?token=${$.cookie('token')}&positionId=${d.MemberId}&recursive=true`,
                type: 'get',
                success: function (res) {
                    if (res.result == 0) {
                        members.concat(res.data.filter(_ => _.Status == 0).map(function (m) {
                            return {
                                Account: d.Account,
                                Name: d.Name,
                                Email: d.Email
                            };
                        }));
                    }
                }
            });
        });

        $.each(distributionUnitList.filter(_ => _.MemberType == 5), function (i, d) {  // 部门
            iso.dmsAjax({
                url: `OrgDepartment/GetUserListByDepartmentId?token=${$.cookie('token')}&departmentId=${d.MemberId}&recursive=true`,
                type: 'get',
                success: function (res) {
                    if (res.result == 0) {
                        members.concat(res.data.filter(_ => _.Status == 0).map(function (m) {
                            return {
                                Account: d.Account,
                                Name: d.Name,
                                Email: d.Email
                            };
                        }));
                    }
                }
            });
        });

        let obj = {};
        members = members.reduce(function (item, next) {
            obj[next.Account] ? '' : obj[next.Account] = true && item.push(next);
            return item;
        }, []);

        return members;
    };

    /**
     *  判断打印文件扩展名是否支持 
     *  @param fileName 文件名称
     */
    checkPrintExtension(fileName) {
        let index = fileName.lastIndexOf('.');
        let extension = fileName.substr(index, fileName.length - index);
        let extensions = ['.doc', '.docx', '.xls', '.xlsx', '.pdf'];

        if (extensions.indexOf(extension) < 0) {
            iso.layerAlert('error', `当前打印仅支持扩展名为${extensions.join('、')}的文件`);
            return false;
        }

        return true;
    };

    /**
     *  获取数组分页
     *  @param cachedName 缓存名称
     *  @param arr 需要做分页的数组
     *  @param pageSize 每页大小
     *  @param pageIndex 第几页
     *  @param recalculate 重新计算
     */
    getArrayPage(cachedName, arr, pageSize, pageIndex, recalculate = false) {
        if (!iso[cachedName] || iso[cachedName].pageSize !== pageSize || recalculate) {
            iso[cachedName] = {
                pageSize: pageSize,
                list: []
            };
        } else {
            return iso[cachedName].list.find(_ => _.pageIndex === pageIndex).list;
        }

        let totalPages = arr.length ? Math.ceil(arr.length / pageSize) : 0;
        if (totalPages === 1) {
            return arr;
        }

        for (var i = 1; i <= totalPages; i++) {
            iso[cachedName].list.push({
                pageIndex: i,
                list: arr.splice(0, pageSize)
            });
        }

        return iso[cachedName].list.find(_ => _.pageIndex === pageIndex).list;
    };

    showDialog(options, callback, closeCallback) {
        let buttons = [];
        if (callback) {
            buttons = [{
                text: '保存',
                handler: function () {
                    let childForm = dialog.iframe[0].contentWindow.iso.eform;
                    if (!childForm.formEngine.validate()) return false;

                    childForm.formEngine.save(function () {
                        if (callback) {
                            callback(dialog);
                        }
                    });
                }
            }];
        }

        let config = {
            width: options.width,
            height: options.height,
            title: options.title,
            maximizable: true,
            isNewUi: true,
            buttons: buttons
        }

        let param2 = {
            dialogType: "iframe",
            target: (options.target.indexOf('&skin=') >= 0 ? options.target : `${options.target}&skin=${window.theme}`),
            showCloseBtn: false
        };
        if ($.isFunction(closeCallback)) {
            param2.onBeforeClose = closeCallback;
        }

        let dialog = new iso.eform.Dialog(config, param2);
    };

    ajax(options) {
        options = $.extend({
            url: '',
            type: 'post',
            data: null,
            dataType: 'json',
            async: true,
            success: function (data) {

            }
        }, options);

        $.ajax({
            url: `${iso.serviceHost}/api/${options.url}`,
            type: options.type,
            data: options.data,
            dataType: options.dataType,
            contentType: 'application/json',
            async: options.async,
            success: function (data) {
                if ($.isFunction(options.success)) {
                    options.success(data);
                }
            },
            beforeSend: function (XMLHttpRequest) {
                XMLHttpRequest.setRequestHeader('Authorization', $.cookie('token'));
                if ($.isFunction(options.beforeSend)) {
                    options.beforeSend(XMLHttpRequest);
                }
            },
            complete: function () {
                if ($.isFunction(options.complete)) {
                    options.complete();
                }
            },
            error: function (a, b, c) {
                iso.layerAlert('error', `网络错误`);  // 网络错误
            }
        });
    };

    dmsAjax(options) {
        let requestParams = {
            url: `${location.origin}/api/services/${options.url}`,
            type: options.type,
            dataType: 'json',
            contentType: 'application/json',
            async: (options.async === false ? false : true),
            success: function (res) {
                if (options.success && $.isFunction(options.success)) {
                    options.success(res);
                }
            },
            error: function () {
                $.messager.layer({
                    type: 'error',
                    content: '网络错误'
                });
            }
        }
        if (options.data) {
            requestParams.data = options.data;
        }

        $.ajax(requestParams);
    };
}

/** 数据集类
 */
class ISODataSet {
    constructor() {
        // ..todo
    };

    /** 
     *  更新基础配置
     * @param recordId 配置项记录Id
     * @param value 配置项json数据
     */
    updateBasicData(recordId, value, text) {
        let params = {
            recordId: recordId,
            value: value,
            text: text,
            modifyBy: iso.eform.userInfo.id,
            modifiedTime: iso.getCurrentDateTime()
        };
        iso.eform.dataset('DB_UpdateBasicData', params, function (res) { }, false);
    };

    /**
     *  根据code查询基础配置
     *  
     *  @param code 配置编号
     *  @param async 是否异步执行，false返回有效value，true必须加上回调
     *  @param callback 回调
     */
    getBasicData(code, async, callback) {
        let value;
        iso.eform.dataset('DB_GetBasicDataByCode', { code: code }, function (res) {
            if (res.Data[0].length > 0 && res.Data[0][0].Value) {
                value = JSON.parse(res.Data[0][0].Value);

                if ($.isFunction(callback)) {
                    callback(value);
                }
            }
        }, async);

        return value;
    };

    /**
     *  根据部门Id，查询部门领导
     *  @param deptId 部门Id
     */
    getLeaderByDeptId(deptId) {
        let leaders;
        iso.eform.dataset('DB_GetDeptLeaderByDeptId', { deptId: deptId }, function (res) {
            if (res.ResultCode == 0) {
                leaders = res.Data[0];
            }
        }, false);

        return leaders.filter(_ => _.userName != '权限管控');
    };

    /**
     *  获取部门|主职位领导
     *  @param type Department or Position
     *  @param id 职位Id||部门id
     */
    getLeaders(type, id) {
        let name = type === 'Department' ? 'DB_GetDeptLeaderByUserId' : 'DB_GetPositionLeaderByUserId';

        let leaders = [];
        iso.eform.dataset(name, {
            id: id
        }, function (res) {
            if (res.ResultCode == 0) {
                leaders = res.Data[0];
            }
        }, false);

        return leaders.filter(_ => _.userName != '权限管控');
    };

    /**
     *  获取DCC文控管理员 
     */
    getDCCManagers() {
        let managers = [];

        iso.eform.dataset('DB_GetGroupMembers', {
            groupName: 'DCC文控管理'
        }, function (res) {
            if (res.ResultCode == 0) {
                managers = res.Data[0];
            }
        }, false);

        return managers.filter(_ => _.userName != '权限管控');
    };

    /**
     *  根据group_code查询用户组信息
     */
    getGroupByCode(groupCode) {
        let group;

        iso.eform.dataset('DB_GetDCCManagerGroup', {
            groupCode: groupCode
        }, function (res) {
            if (res.ResultCode === 0) {
                group = res.Data[0][0];
            }
        }, false);

        return group;
    }

    /**
     *  查询顶级部门Id 
     */
    getTopDeptId() {
        let deptId = 0;
        iso.eform.dataset('DB_GetTopDept', {}, function (res) {
            if (res.ResultCode == 0 && res.Data[0].length > 0) {
                deptId = res.Data[0][0].DeptId;
            }
        }, false);

        return deptId;
    };

    /**
     *  查询顶级职位Id 
     */
    getTopPositionId() {
        let positionId = 0;
        iso.eform.dataset('DB_GetTopPosition', {}, function (res) {
            if (res.ResultCode == 0 && res.Data[0].length > 0) {
                positionId = res.Data[0][0].PositionId;
            }
        }, false);

        return positionId;
    };

    /**
     *  根据用户Id，获取用户信息 
     *  @param userId 用户Id
     */
    getUserInfoById(userId) {
        let user;
        iso.eform.dataset('DB_GetUserById', { id: userId }, function (res) {
            if (res.ResultCode == 0) {
                user = res.Data[0][0];
            }
        }, false);

        return user;
    };

    /**
     *  获取最新流程设计信息 
     *  @param 流程名称
     */
    getNewestWorkflow(name) {
        let result;

        iso.eform.dataset('DB_GetNewestWorkflow', {
            name: name
        }, function (res) {
            result = res.Data[0];
        }, false);

        return result;
    };

    /**
     *  获取关联文件
     *  @param documentFlowId 文档流程Id
     */
    getAssociatedFiles(documentFlowId) {
        let associatedFiles = [];
        iso.eform.dataset('DB_GetAssociatedFiles', {
            referenceId: documentFlowId
        }, function (res) {
            if (res.ResultCode == 0) {
                associatedFiles = res.Data[0];
            }
        }, false);

        return associatedFiles;
    };

    /**
     *  根据文件夹名称查询文件夹信息
     *  @param folderName 文件夹名称
     */
    getFolderByName(folderName) {
        let result;
        iso.eform.dataset('DB_GetFolderByName', {
            folderName: folderName
        }, function (res) {
            if (res.ResultCode == 0) {
                result = res.Data[0][0];
            }
        }, false);

        return result;
    };

    /**
     *  获取服务配置 
     */
    getServiceCongfigure() {
        let config;
        iso.eform.dataset('DB_GetServiceConfigure', {}, function (res) {
            if (res.ResultCode == 0 && res.Data[0].length > 0) {
                config = res.Data[0][0];
            }
        }, false);

        return config;
    };

    /**
     *  查询静态数据源
     *  @param dataSourceName 数据源名称
     */
    getStaticDataSource(dataSourceName) {
        eform.dataset('', {

        });
    };

    /**
     *  添加培训 
     *  @param fileId 文件Id
     */
    addTraining(fileId) {
        let result;

        eform.dataset('DB_GetFileInfoByFileId', {
            fileId: fileId
        }, function (res) {
            if (res.ResultCode != 0) {
                result = res;
                return res;
            }

            let data = res.Data[0][0];
            eform.dataset('DB_AddTraining', {
                id: $.genId(),
                createId: eform.userInfo.id,
                createTime: iso.getCurrentDateTime(),
                fileInfoId: data.Id,
                trainingLeaderId: data.TrainingLeaderId,
                trainingLeaderName: data.TrainingLeaderName
            }, function (res) {
                result = res;
            }, false);
        }, false);

        return result;
    };

    /**
     *  distributionFile添加触发器
     */
    initDistributionFile() {
        iso.eform.dataset('DB_ExecSql', {}, function (res) { }, false);
    };

    /**
     *  查询签收规则：0自动签收、1手动签收
     */
    getSignInRule() {
        let result;
        iso.eform.dataset('DB_GetSignInRule', {}, function (res) {
            result = res;
        }, false);

        return result;
    };

    /**
     * 根据referenceId，查询分配人员信息
     * @param referenceId 资源Id
     */
    getDistributionMembersByReferenceId(referenceId) {
        let members = [];
        iso.eform.dataset('DB_GetDistributionUnitByReferenceId', {
            referenceId: referenceId
        }, function (res) {
            if (res.ResultCode == 0 && res.Data[0].length > 0) {
                members = isoDataSet.getDistributionMembers(res.Data[0]);
            }
        }, false);

        return members;
    };

    /**
     *  获取分配对象人员信息
     *  @param {Object[]} distributions 分配对象
     *  @param {String} distributions[].MemberId 对象Id
     *  @param {String} distributions[].MemberName 对象名称
     *  @param {String} distributions[].MemberType 对象类型
     */
    getDistributionMembers(distributions) {
        let members = [];
        let userIds = distributions.filter(_ => _.MemberType == 0).map(_ => _.MemberId);  // 用户
        if (userIds.length > 0) {
            iso.eform.dataset('DB_QueryUsersInUserIds', {
                edoc2inarea_userIds: userIds.join()
            }, function (res) {
                if (res.ResultCode == 0 && res.Data[0].length > 0) {
                    members = res.Data[0].map((d) => {
                        return {
                            Account: d.Account,
                            Name: d.Name,
                            Email: d.Email
                        };
                    });
                }
            }, false);
        }

        let groups = distributions.filter(_ => _.MemberType == 3);
        if (groups.filter(_ => _.MemberName == 'Everyone').length > 0) {
            iso.eform.dataset('DB_QueryValidUsers', {}, function (res) {
                if (res.ResultCode == 0 && res.Data[0].length > 0) {
                    members = members.concat(res.Data[0].map((d) => {
                        return {
                            Account: d.Account,
                            Name: d.UserName,
                            Email: d.Email
                        };
                    }));
                }
            }, false);
        } else {
            let groupIds = distributions.filter(_ => _.MemberType == 3).map(_ => _.MemberId);  // 用户组
            if (groupIds.length > 0) {
                iso.eform.dataset('DB_GetMembersByGroupIds', {
                    edoc2inarea_groupIds: groupIds.join()
                }, function (res) {
                    if (res.ResultCode == 0 && res.Data[0].length > 0) {
                        members = members.concat(res.Data[0].map((d) => {
                            return {
                                Account: d.Account,
                                Name: d.UserName,
                                Email: d.Email
                            };
                        }));
                    }
                }, false);
            }
        }

        let positionIds = distributions.filter(_ => _.MemberType == 4).map(_ => _.MemberId);  // 职位
        if (positionIds.length > 0) {
            $.each(positionIds, function (i, positionId) {
                iso.dmsAjax({
                    url: `OrgPosition/GetUserListByPositionId?token=${$.cookie('token')}&positionId=${positionId}&recursive=true`,
                    type: 'get',
                    async: false,
                    success: function (res) {
                        if (res.result == 0 && res.data.length > 0) {
                            members = members.concat(res.data.map((d) => {
                                return {
                                    Account: d.Account,
                                    Name: d.Name,
                                    Email: d.Email
                                };
                            }));
                        }
                    }
                });
            });
        }

        let deptIds = distributions.filter(_ => _.MemberType == 5).map(_ => _.MemberId);  // 部门
        if (deptIds.length > 0) {
            $.each(deptIds, function (i, deptId) {
                iso.dmsAjax({
                    url: `OrgDepartment/GetUserListByDepartmentId?token=${$.cookie('token')}&departmentId=${deptId}&recursive=true`,
                    type: 'get',
                    async: false,
                    success: function (res) {
                        if (res.result == 0 && res.data.length > 0) {
                            members = members.concat(res.data.map((d) => {
                                return {
                                    Account: d.Account,
                                    Name: d.Name,
                                    Email: d.Email
                                };
                            }));
                        }
                    }
                });
            });
        }

        if (members.length > 0) {
            let obj = {};
            members = members.reduce(function (item, next) {
                obj[next.Account] ? '' : obj[next.Account] = true && item.push(next);
                return item;
            }, []);
        }

        return members;
    };

    /**
     *  新增打印记录 
     *  @param printResult 打印结果
     *  @param {String} printResult.printTaskId  打印任务Id
     *  @param {String} printResult.printStatus  打印状态
     */
    insertPrintRecord(printResult) {
        let createTime = iso.getCurrentDateTime();
        let param = {
            id: $.genId(),
            createTime: createTime,
            modifiedTime: createTime,
            createId: iso.eform.userInfo.id,
            updateId: iso.eform.userInfo.id,
            printTaskId: printResult.PrintTaskId,
            printBarcode: Math.round(new Date()),  // 随机条码
            printStatus: printResult.PrintStatus,
            message: printResult.Message ?? '',
            fileInfoId: printResult.FileInfoId,
            numberOfApplications: printResult.NumberOfApplications,  // 申请份数
            copyCount: printResult.CopyCount,  // 打印份数
            distributionStatus: 0,  // 下发状态
            printer: printResult.Printer,
            printerName: printResult.PrinterName,
            printTime: printResult.PrintTime,
            issued: 0,  // 是否已发放
            recycled: 0,  // 是否已回收
            destroyed: 0,  // 是否已销毁
            recyclingStatus: 0  // 回收状态
        };

        iso.eform.dataset('DB_InsertPrintRecord', param, function (res) {
            if (res.ResultCode != 0) {
                return iso.layerAlert('error', '新增打印记录失败');
            }

            return iso.layerAlert('success', `打印${iso.CNPrintStatus[printResult.PrintStatus]}`);
        }, false);

        iso.eform.dataset('DB_UpdatePrintStatus', {
            id: printResult.PrintTaskId,
            printStatus: 1
        }, function (res) {
            if (iso.eform('eformDataGrid1')) {
                iso.eform('eformDataGrid1').method('load');
            }
        }, false);
    }

    /**
     *  查询打印管理员 
     */
    queryPrintAdministrator() {
        let result;

        iso.eform.dataset('DB_QueryPrinters', {}, function (res) {
            result = res;
        }, false);

        return result;
    }
};

/** IO类
 */
class IO {
    CachedToken = {};

    /**
     *  用户类型，key: eform.memberType, value: ecm.memberType
     */
    MemberTypeObj = {
        0: 1,  // 用户
        3: 8,  // 用户组
        4: 4,  // 职位
        5: 2   // 部门
    };

    /**
     *  用户类型，key: eform.memberType, value: ecm.memberType
     */
    CNMemberTypeObj = {
        0: '用户',
        3: '用户组',
        4: '职位',
        5: '部门'
    };

    /**
     *  文件夹权限列表
     */
    PermissionList = [];

    /**
     *  版本策略 
     */
    VersionStrategy = {
        Minor: 0,  // 次版本
        Major: 1  // 主版本
    };

    constructor() {
        this.TempFolder = 17;
    };

    /**
     *  获取权限名称
     *  @param 权限类别Id
     */
    getPermissionName(permCateId) {
        if (this.PermissionList.length === 0) {
            this.getPermCategoryList(1, function (data) {
                isoIO.PermissionList = data;
            }, false);
        }

        let perm = this.PermissionList.filter(_ => _.CateId == permCateId)[0];
        if (perm) {
            return perm.Name;
        }

        return '';
    };

    /**
     *  获取权限分类Id 
     *  @param 权限名称
     */
    getPermissionCategoryId(permName) {
        if (this.PermissionList.length === 0) {
            this.getPermCategoryList(1, function (data) {
                isoIO.PermissionList = data;
            }, false);
        }

        let perm = this.PermissionList.filter(_ => _.Name == permName)[0];
        if (perm) {
            return perm.CateId;
        }

        return 0;
    };

    /**
     *  获取缓存token（根据继承登录获取的token）
     *  @param account 用户账号
     */
    getCachedToken(account) {
        if (!this.CachedToken[account] || !isoIO.checkTokenValidity(this.CachedToken[account])) {
            isoIO.userLoginIntegrationByUserLoginName(account, function (res) {
                if (res.ErrCode != 200 || res.Data.result != 0) {
                    return;
                }

                isoIO.CachedToken[account] = res.Data.data;
            });
        }

        return isoIO.CachedToken[account];
    };

    /** 
     *  获取权限列表
     * @param entryRype 文档类型：1、文件夹 2、文件
     */
    getPermCategoryList(entryRype, callback, async) {
        iso.ajax({
            url: `Doc/GetPermCategoryList?entryType=${entryRype}`,
            type: 'get',
            async: (async === false ? false : true),
            success: function (res) {
                if (res.ErrCode !== 200 || res.Data.result !== 0) {
                    return iso.layerAlert('error', '获取权限列表失败');  // 获取权限列表失败
                }

                if ($.isFunction(callback) && res.Data.data) {
                    callback(res.Data.data);
                }
            }
        });
    };

    /** 
     *  创建文件夹
     *  @param folderName 文件夹名称
     *  @param parentFolderId 父级文件夹Id
     */
    createFolder(token, folderName, parentFolderId, callback) {
        iso.ajax({
            url: `Folder/CreateFolder`,
            data: JSON.stringify({
                Token: token,
                Name: folderName,
                ParentFolderId: parseInt(parentFolderId)
            }),
            async: false,
            success: function (res) {
                if (res.ErrCode !== 200 || (res.Data.result !== 0 && res.Data.result != 806)) {
                    parent.iso.layerAlert('error', '创建文件夹失败');  // 创建文件夹失败
                }

                if ($.isFunction(callback) && res.Data.data) {
                    callback(res.Data);
                }
            }
        });
    };

    /**
     *  文件夹重命名
     *  @param folderId 文件夹Id
     *  @param folderName 文件夹名称
     */
    renameFolder(folderId, folderName) {
        iso.ajax({
            url: 'Folder/RenameFolder',
            data: JSON.stringify({
                Token: $.cookie('token'),
                DocId: parseInt(folderId),
                NewName: folderName
            }),
            async: false,
            success: function (res) {
                if (res.ErrCode != 200 || res.Data.result != 0) {
                    return iso.layerAlert('error', '文件夹重命名失败');
                }

                iso.layerAlert('success', '文件夹重命名成功');
            }
        });
    }

    /** 
     *  设置文件夹权限，根class 限类别Id
     * @pam folderId 文件夹Id
     * @param catd 权限类别Id
     * @param memberType 成员类型：1 用户 2 部 4 职位 8 用户组
     */
    setFolderPermissionByCateId(folderId, cateId, memberId, memberType, callback) {
        iso.ax({
            url: 'Folder/SetFolderPermissionByCateId',
            data: JSON.stringify({
                FolderId: parseInt(folderId),
                CateId: cateId,
                MemberId: memberId,
                MemberType: memberType
            }),
            async: false,
            success: function (res) {
                if (res.ErrCode !== 200 || res.Data.result !== 0) {
                    return iso.layerAlert('error', '绑定权限失败');  // 绑定权限失败
                }

                if ($.isFunction(callback)) {
                    callback(res.Data.data);
                }
            }
        });
    };

    /** 
     *  设置文件夹权限
     * @param folderId 文件夹Id
     * @param token 分配权限的token
     * @param {Object[]} permissions 权限列表
     * @param {Number} permissions[].MemberId 成员Id
     * @param {Number} permissions[].MemberType 成员类型
     * @param {Number} permissions[].PermCateId 权限类别Id
     */
    setPermissions(folderId, token, permissions, callback) {
        token = isoIO.getCachedToken('rightsmanager');  // 文档权限管理者

        iso.ajax({
            url: 'FolderPermission/SetPermissions',
            data: JSON.stringify({
                Token: token,
                ID: parseInt(folderId),
                Permissions: permissions
            }),
            async: false,
            success: function (res) {
                if ($.isFunction(callback)) {
                    callback(res);
                }
            }
        });
    };

    /**
     *  清空指定文件夹权限
     *  @param folderId 文件夹Id
     */
    clearPermissionsByFolderId(folderId, callback) {
        let result = true;

        iso.ajax({
            url: `FolderPermission/GetFolderPermission?folderId=${folderId}`,
            type: 'get',
            async: false,
            success: function (res) {
                if (res.ErrCode != 200 || res.Data.result != 0) {
                    result = false;
                    return;
                }

                let permissions = res.Data.data.FolderPermissionList.filter(_ => _.permType == '10').map(function (d) {
                    return {
                        MemberId: d.memberId,
                        MemberType: d.memberType,
                        PermType: 10
                    };
                });

                let delResult = isoIO.deleteFolderPermission(folderId, permissions);
                if (delResult.ErrCode != 200 || delResult.Data.result != 0) {
                    result = false;
                }
            }
        });

        return result;
    };

    /**
     *  删除指定文件夹，指定对象的权限
     *  @param folderId 文件夹Id
     *  @param permissions 
     */
    deleteFolderPermission(folderId, permissions) {
        let result;

        iso.ajax({
            url: 'FolderPermission/DeleteFolderPermission',
            data: JSON.stringify({
                Token: isoIO.getCachedToken('rightsmanager'),
                FolderId: parseInt(folderId),
                Mermbers: permissions
            }),
            async: false,
            success: function (res2) {
                result = res2;
                if (res2.ErrCode != 200 || res2.Data.result != 0) {
                    result = false;
                    return;
                }
            }
        });

        return result;
    };

    /**
     *  删除文件和文件夹列表 
     *  @param fileIdList 文件Ids
     *  @param folderIdList 文件夹Ids
     */
    removeFolderListAndFileList(fileIdList, folderIdList, callback) {
        iso.ajax({
            url: 'Doc/RemoveFolderListAndFileList',
            type: 'post',
            data: JSON.stringify({
                Token: isoIO.getCachedToken('rightsmanager'),
                FileIdList: fileIdList,
                FolderIdList: folderIdList
            }),
            async: false,
            success: function (res) {
                if (res.ErrCode !== 200 || res.Data.result !== 0) {
                    return iso.layerAlert('error', '删除文件和文件夹列表失败');  // 删除文件和文件夹失败
                }

                if ($.isFunction(callback)) {
                    callback(res.Data);
                }
            }
        });
    };

    /**
     *  剪切文件与文件夹
     *  @param targetFolderId 目标文件夹id
     *  @param fileIdList 文件Ids
     *  @param folderIdList 文件夹ids
     */
    moveFolderListAndFileList(token, targetFolderId, fileIdList, folderIdList, callback) {
        iso.ajax({
            url: 'Doc/MoveFolderListAndFileList',
            type: 'post',
            data: JSON.stringify({
                Token: token,
                TargetFolderId: parseInt(targetFolderId),
                FileIdList: fileIdList,
                FolderIdList: folderIdList
            }),
            async: false,
            success: function (res) {
                if (res.ErrCode !== 200 || (res.Data.result !== 0 && res.Data.result != 2007)) {
                    iso.layerAlert('error', '剪切文件与文件夹失败');  // 剪切文件与文件夹失败
                }

                if ($.isFunction(callback)) {
                    callback(res);
                }
            }
        });
    };

    /**
     *  升级主版本号（无升级次版本接口）
     *  @param fileId 文件Id
     */
    publishFileVersion(token, fileId, callback) {
        //token = isoIO.getCachedToken('rightsmanager');  // 文档权限管理者

        iso.ajax({
            url: 'File/PublishFileVersion',
            type: 'post',
            data: JSON.stringify({
                Token: token,
                FileId: parseInt(fileId)
            }),
            async: false,
            success: function (res) {
                if ($.isFunction(callback)) {
                    callback(res);
                }
            }
        });
    };

    /**
     *  逻辑升级主版本 
     *  @param type 版本类型：isoIO.VersionStrategy
     *  @param version 当前版本
     */
    upgradeVersion(type, version) {
        let versions = version.split('.');
        if (type === isoIO.VersionStrategy.Major) {
            let num = parseInt(versions[0]) + 1;
            return `${num}.0`;
        } else {
            let num = parseInt(versions[1]) + 1;
            return `${versions[0]}.${num}`;
        }
    };
    /**
     *  逻辑升级主版本-新
     *  @param type 版本类型：isoIO.VersionStrategy
     *  @param version 当前版本
     *  @param versionType 版本类别
     *  @param initNum 初始版本
     *  @param versionNum 小版本数量
     */
    upgradeVersionNew(type, version, versionType, initNum, versionNum) {
        switch (versionType) {
            case '0':
                let versions = version.split('.');
                if (type === isoIO.VersionStrategy.Major) {
                    let num = parseInt(versions[0]) + 1;
                    return `${num}.0`;
                } else {
                    let num = parseInt(versions[1]) + 1;
                    return `${versions[0]}.${num}`;
                }

                break;
            case '1':
                let letter = version.replace(/[^a-zA-Z]/g, ''),//提取字母
                    num = parseInt(version.replace(/[^\d]/g, ''));//提取数字
                let increaseLetter = letter.charCodeAt(0) >= 90 ? 'Z' : String.fromCharCode(letter.charCodeAt(0) + 1);
                if (parseInt(type) === isoIO.VersionStrategy.Major) {
                    letter = increaseLetter;
                    num = initNum;
                } else {
                    if ((num - parseInt(initNum) + 1) >= parseInt(versionNum)) {

                        num = initNum;
                        letter = increaseLetter;
                    } else {
                        num = num + 1;
                    }
                }
                return letter + num;
                break;
            case '2':
                return String.fromCharCode(version.charCodeAt(0) + 1);
                break;
        }
    }
    /**
     *  回收审批人权限
     *  @description 1、回收审批人权限 2、防止审批人在分发单位内，重新给分发单位下发权限
     *  @param identityIds 审批人identityId
     *  @param {Object[]} permFolders 待分配文档流程Id和文件夹Id
     *  @param {string} permFolders[].folderId 文件夹Id
     *  @param {string} permFolders[].fileId 文件Id
     *  @param actionType 流程事件类型 eform.wf.actionType.cancel || eform.wf.actionType.approve
     *  @param {boolean} distribution 是否给分配单位重新下发权限
     */
    permissionOfRecyclingApprover(identityIds, permFolders, actionType, distribution) {
        let delPermList = {
            FolderId: '',
            Mermbers: identityIds.map(function (identityId) {
                return {
                    MemberId: identityId,
                    MemberType: 1
                }
            })
        };

        let result = true;
        $.each(permFolders, function (i, d) {
            delPermList.Token = isoIO.getCachedToken('rightsmanager');
            delPermList.FolderId = parseInt(d.folderId);

            iso.ajax({
                url: 'FolderPermission/DeleteFolderPermission',
                data: JSON.stringify(delPermList),
                async: false,
                success: function (res) {
                    if (res.ErrCode != 200 || res.Data.result != 0) {
                        result = false;
                        return;
                    }

                    distribution = distribution == undefined ? true : distribution;
                    if (distribution) {  // 是否分配
                        let distributionUnits;
                        eform.dataset('DB_GetDistributionUnitsByFileId', {
                            fileId: d.fileId
                        }, function (res2) {
                            distributionUnits = res2.Data[0];
                        }, false);

                        if (actionType == eform.wf.actionType.cancel) {
                            distributionUnits = distributionUnits.filter(_ => _.Valid == '1');  // 补发分配单位的时候，如果取消，过滤无效分配单位
                        }

                        //let permCateId = isoIO.getPermissionCategoryId('ISO-只读');
                        //if (permCateId === 0) {
                        //    result = false;
                        //    return;
                        //}
                        let setPermList = distributionUnits.map(function (unit) {
                            return {
                                MemberId: unit.MemberId,
                                MemberType: unit.MemberType,
                                Permission: unit.Permission
                            };
                        });
                        eform.dataset('DB_GetApplicationUnit', {
                            fileId: d.fileId,
                            currentTime: iso.getCurrentDateTime()
                        }, function (res) {
                            if (res.ResultCode == 0 && res.Data[0].length > 0) {
                                setPermList = setPermList.concat(res.Data[0]);
                            }
                        }, false);
                        setPermList = iso.getPermissionObject(setPermList);
                        if (setPermList.length === 0) {
                            result = true;
                            return;
                        }

                        let obj = {};
                        setPermList = setPermList.reduce(function (item, next) {
                            obj[next.MemberId] ? '' : obj[next.MemberId] = true && item.push(next);
                            return item;
                        }, []);

                        isoIO.setPermissions(d.folderId, '', setPermList, function (res3) {
                            if (res3.ErrCode != 200 || res3.Data.result != 0) {
                                result = false;
                            }
                        });
                    }
                }
            });

            return result;
        });

        return result;
    };

    /**
     *  集成登录
     *  @param loginName 登录账号
     */
    userLoginIntegrationByUserLoginName(loginName, callback) {
        iso.ajax({
            url: `Org/UserLoginIntegrationByUserLoginName?loginName=${loginName}`,
            type: 'get',
            async: false,
            success: function (res) {
                if ($.isFunction(callback)) {
                    callback(res);
                }
            }
        })
    };

    /**
     *  获取文件信息
     *  @param fileId 文件Id
     */
    getFileInfoById(fileId) {
        let result;
        iso.ajax({
            url: `File/GetFileInfoById?fileId=${fileId}`,
            type: 'get',
            async: false,
            success: function (res) {
                result = res;
            }
        });

        return result;
    };

    /**
     *  验证token是否有效
     */
    checkTokenValidity(token) {
        let result = true;
        iso.ajax({
            url: `Org/CheckUserTokenValidity?token=${token}`,
            type: 'get',
            async: false,
            success: function (res) {
                if (res.ErrCode != 200 || res.Data.result != 0) {
                    result = false;
                }
            }
        });

        return result;
    };
};

window.customEformJsCallbackList.push(function (eform, edoc2Form, formParser) {
    window.iso = new ISO(eform);
    window.isoIO = new IO();
    window.isoDataSet = new ISODataSet();
});